<?php
namespace Ep\Gath\Form;

use Laminas\Form\Form;
use Laminas\Stdlib\InitializableInterface;

/**
 * Class MotorcycleSelect
 *
 * @package Gath\Moto\Form
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/gath-moto
 */
class SelectMotorcycle extends Form implements InitializableInterface
{
    public function init()
    {
        parent::init();

        $this->setAttributes([
            'class' => 'select-motorcycle__form',
        ]);

        $this->add([
            'type' => 'gath.motorcycle_brand_select',
            'name' => 'brandId',
            'options' => [
                'label' => 'Proizvođač',
            ],
            'attributes' => [
                'data-selectize' => true,
            ],
        ]);

        $this->add([
            'type' => 'gath.motorcycle_model_select',
            'name' => 'modelId',
            'options' => [
                'label' => 'Model',
            ],
            'attributes' => [
                'data-selectize' => true,
            ],
        ]);

        $this->add([
            'type' => 'gath.motorcycle_select',
            'name' => 'motorcycleId',
            'options' => [
                'label' => 'Godina',
            ],
            'attributes' => [
                'data-selectize' => true,
            ],
        ]);

        $this->add([
            'type' => 'submit',
            'name' => 'select-motorcycle',
            'options' =>  [
                'label' => 'Traži',
            ],
        ]);

        $filter = $this->getInputFilter();
        $filter->add([
            'name' => 'brandId',
            'required' => true,
        ]);
    }

    public function setData($data)
    {
        if (isset($data['brandId']) && !empty($data['brandId'])) {
            /** @var Element\MotorcycleModelSelect $model */
            $model = $this->get('modelId');
            $model->setBrandId(intval($data['brandId']));
        }

        if (isset($data['modelId']) && !empty($data['modelId'])) {
            /** @var Element\MotorcycleSelect $motorcycle */
            $motorcycle = $this->get('motorcycleId');
            $motorcycle->setModelId(intval($data['modelId']));
        }

        if (isset($data['motorcycleId']) && !empty($data['motorcycleId'])) {
            $this->add([
                'type' => 'submit',
                'name' => 'reset-motorcycle',
                'attributes' => [
                    'class' => 'select-motorcycle__reset',
                ],
                'options' => [
                    'icon' => 'ban',
                ],
            ]);
        }

        return parent::setData($data);
    }
}
