<?php
namespace Ep\Gath\Plugin;

/**
 * Koristimo stanja zaliha proizvoda iz Gatha.
 *
 * @package Ep\Gath\Plugin
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/gath
 */
class Stock extends AbstractGathPlugin
{
    /** @var array */
    protected $resultsById = [];
    /** @var array */
    protected $queuedIds = [];
    /** @var array */
    protected $ignoredIds = [];

    /**
     * Dodaje akcije i filtere u WordPress.
     *
     * @return void
     */
    public function attach(): void
    {
        add_filter('get_post_metadata', [$this, 'get_post_metadata'], 10, 3);
        add_filter('the_posts', [$this, 'the_posts'], 10);
        add_filter('ep_gath_stock', [$this, 'ep_gath_stock']);
    }

    public function ep_gath_stock()
    {
        return $this;
    }

    /**
     * Ako je netko dohvatio proizvode iz Gatha, zapamtiti ID-eve jer ćemo
     * trebat vjerovatno dohvatiti cijene nekad.
     *
     * @param \WP_Post[] $posts
     *
     * @return mixed
     */
    public function the_posts($posts)
    {
        if (is_array($posts)) {
            foreach ($posts as $post) {
                if (!in_array($post->post_type, ['product', 'product_variation'])) {
                    continue;
                }

                $this->queuedIds[] = $post->ID;
            }
        }

        return $posts;
    }

    public function get_post_metadata($null, $object_id, $meta_key)
    {
        if (!in_array($meta_key, [
            '_manage_stock',
            '_purchase_step',
            '_stock',
            '_stock_status',
            '_stock_by_warehouses',
        ]) || in_array($object_id, $this->ignoredIds)) {
            return $null;
        }

        if (!isset($this->resultsById[$object_id])) {
            $this->queuedIds[] = $object_id;
            $this->load();
        }

        if (!isset($this->resultsById[$object_id])) {
            return $null;
        }

        $result = $this->resultsById[$object_id];
        $return = '';

        switch ($meta_key) {
            case '_manage_stock':
                return 'yes';
            case '_purchase_step':
                return $result['PurchaseStep'] ?? 0;
            case '_stock':
                $return = $this->getTotalStock($result);
                break;
            case '_stock_status':
                $return = $this->getTotalStock($result) > 0 ? 'instock' : 'outofstock';
                break;
            case '_stock_by_warehouses':
                return $this->getStockByWarehouses($result);
        }

        return $return;
    }

    private function getTotalStock(array $result): int
    {
        $config = $this->gath->getConfig();

        if (!isset($config['working_stock']) || !is_array($config['working_stock']) || empty($config['working_stock'])) {
            return $result['Stock'];
        }

        $stock = 0;

        foreach ($config['working_stock'] as $fields) {
            foreach ($fields as $field) {
                $key = "Stock_$field";
                if (isset($result[$key])) {
                    $stock += $result[$key];
                }
            }
        }

        return $stock;
    }

    private function getStockByWarehouses(array $result): array
    {
        $stock = [];

        foreach ($result as $key => $value) {
            if (preg_match('/^(Stock_([0-9]+)|PartnerStock)$/', $key, $matches)) {
                $stock[$key] = $value;
            }
        }

        return $stock;
    }

    protected function load(): void
    {
        if (count($this->queuedIds) === 0) {
            return;
        }

        $skus = [];
        $tireIds = [];

        foreach ($this->queuedIds as $preloadId) {
            if (isset($this->resultsById[$preloadId]) || in_array($preloadId, $this->ignoredIds)) {
                continue;
            }

            if ($preloadId > Products::ID_OFFSET_GROUPED && $preloadId < Products::ID_OFFSET_VARIATION) {
                $this->ignoredIds[] = $preloadId;
                continue;
            }

            if ($preloadId > Products::ID_OFFSET_TIRES) {
                $tireIds[$preloadId] = (int)($preloadId - Products::ID_OFFSET_TIRES);
                continue;
            }

            $skus[$preloadId] = get_post_meta($preloadId, '_sku', true);
        }

        if (count($tireIds) > 0 || count($skus) > 0) {
            $data = $this->gath->getStockBySkus(array_values($skus), array_values($tireIds));

            foreach ($data['Results'] as $result) {
                $productId = array_search($result['ProductCode'], $skus);

                if (!$productId)
                    $productId = array_search($result['ProductId'], $tireIds);

                $this->resultsById[$productId] = $result;
            }
        }

        $this->queuedIds = [];
    }
}
